@@ -19,6 +19,16 @@
 
 ;;; History:
 
+;; Version 2.3.0:
+;;   * New =<feed-matcher> syntax in search filters
+;;   * Support for protocol-relative entry links (i.e. //example.com/foo/)
+;;   * New `elfeed-add-feed' :save key argument
+;;   * New plist-based parsed search filter format
+;;   * New hook: `elfeed-search-update-hook'
+;;   * New hook: `elfeed-db-unload-hook'
+;;   * New variable: `elfeed-search-sort-function'
+;;   * Connect curl with a pipe instead of a pty: performance boost
+;;   * Minor bug fixes
 ;; Version 2.2.0:
 ;;   * Support for org links (elfeed-link.el)
 ;;   * Added `elfeed-db-unload'
@@ -140,7 +150,7 @@
   "An Emacs web feed reader."
   :group 'comm)
 
-(defconst elfeed-version "2.2.0")
+(defconst elfeed-version "2.3.0")
 
 (defcustom elfeed-feeds ()
   "List of all feeds that Elfeed should follow.
@@ -337,9 +347,17 @@ Take three arguments: the feed TYPE, the XML structure for the
 entry, and the Elfeed ENTRY object. Return value is ignored, and
 is called for side-effects on the ENTRY object.")
 
+(defsubst elfeed--fixup-protocol (protocol url)
+  "Prepend PROTOCOL to URL if it is protocol-relative.
+If PROTOCOL is nil, returns URL."
+  (if (and protocol url (string-match-p "^//[^/]" url))
+      (concat protocol ":" url)
+    url))
+
 (defun elfeed-entries-from-atom (url xml)
   "Turn parsed Atom content into a list of elfeed-entry structs."
   (let* ((feed-id url)
+         (protocol (url-type (url-generic-parse-url url)))
          (namespace (elfeed-url-to-namespace url))
          (feed (elfeed-db-get-feed feed-id))
          (title (elfeed-cleanup (xml-query* (feed title *) xml)))
@@ -355,8 +373,10 @@ is called for side-effects on the ENTRY object.")
                                xml-base (xml-query* (:base) (list entry))))
                     (anylink (xml-query* (link :href) entry))
                     (altlink (xml-query* (link [rel "alternate"] :href) entry))
-                    (link (elfeed-update-location
-                           xml-base (or altlink anylink)))
+                    (link (elfeed--fixup-protocol
+                           protocol
+                           (elfeed-update-location xml-base
+                                                   (or altlink anylink))))
                     (date (or (xml-query* (published *) entry)
                               (xml-query* (updated *) entry)
                               (xml-query* (date *) entry)
@@ -407,6 +427,7 @@ is called for side-effects on the ENTRY object.")
 (defun elfeed-entries-from-rss (url xml)
   "Turn parsed RSS content into a list of elfeed-entry structs."
   (let* ((feed-id url)
+         (protocol (url-type (url-generic-parse-url url)))
          (namespace (elfeed-url-to-namespace url))
          (feed (elfeed-db-get-feed feed-id))
          (title (elfeed-cleanup (xml-query* (rss channel title *) xml)))
@@ -416,7 +437,9 @@ is called for side-effects on the ENTRY object.")
     (cl-loop for item in (xml-query-all* (rss channel item) xml) collect
              (let* ((title (or (xml-query* (title *) item) ""))
                     (guid (xml-query* (guid *) item))
-                    (link (or (xml-query* (link *) item) guid))
+                    (link (elfeed--fixup-protocol
+                           protocol
+                           (or (xml-query* (link *) item) guid)))
                     (date (or (xml-query* (pubDate *) item)
                               (xml-query* (date *) item)))
                     (author (or (xml-query* (author *) item)
@@ -584,8 +607,10 @@ Only a list of strings will be returned."
        nil))
     (nreverse res)))
 
-(defun elfeed-add-feed (url)
-  "Manually add a feed to the database."
+(cl-defun elfeed-add-feed (url &key save)
+  "Manually add a feed to the database.
+If SAVE is non-nil the new value of ‘elfeed-feeds’ is saved.  When
+called interactively, SAVE is set to t."
   (interactive
    (list
     (let* ((feeds (elfeed-candidate-feeds))
@@ -595,9 +620,10 @@ Only a list of strings will be returned."
            (result (elfeed-cleanup input)))
       (cond ((not (zerop (length result))) result)
             (feeds (car feeds))
-            ((user-error "No feed to add"))))))
-  (cl-pushnew url elfeed-feeds)
-  (when (called-interactively-p 'any)
+            ((user-error "No feed to add"))))
+    :save t))
+  (cl-pushnew url elfeed-feeds :test #'equal)
+  (when save
     (customize-save-variable 'elfeed-feeds elfeed-feeds))
   (elfeed-update-feed url))
 
@@ -718,8 +744,10 @@ saved to your customization file."
 (provide 'elfeed)
 
 (cl-eval-when (load eval)
-  (require 'elfeed-csv)
-  (require 'elfeed-show)
-  (require 'elfeed-search))
+  ;; run-time only, so don't load when compiling other files
+  (unless byte-compile-root-dir
+    (require 'elfeed-csv)
+    (require 'elfeed-show)
+    (require 'elfeed-search)))
 
 ;;; elfeed.el ends here
